class TestRunnerTags:
    TEST_RUN_ID = "test.run.id"
    TEST_RUN_TASK_ID = "test.run.task.id"
    TEST_SUITE_TRANSACTION_ID = "test.suite.transaction.id"
    TEST_NAME = "test.name"
    TEST_SUITE = "test.suite"
    TEST_FIXTURE = "test.fixture"
    TEST_STATUS = "test.status"
    TEST_SUITE_TOTAL_COUNT = "test.suite.total.count"
    TEST_SUITE_SUCCESSFUL_COUNT = "test.suite.successful.count"
    TEST_SUITE_FAILED_COUNT = "test.suite.failed.count"
    TEST_SUITE_ABORTED_COUNT = "test.suite.aborted.count"
    TEST_SUITE_SKIPPED_COUNT = "test.suite.skipped.count"
    TEST_METHOD = "test.method"
    TEST_CLASS = "test.class"
    TEST_TIMEOUT = "test.timeout"
    TEST_BEFORE_EACH_DURATION = "test.before.each.duration"
    TEST_AFTER_EACH_DURATION = "test.after.each.duration"
    TEST_BEFORE_ALL_DURATION = "test.before.all.duration"
    TEST_AFTER_ALL_DURATION = "test.after.all.duration"
    TEST_ENVIRONMENT = "test.env"
    SOURCE_CODE_REPO_URL = "src.repo.url"
    SOURCE_CODE_REPO_NAME = "src.repo.name"
    SOURCE_CODE_BRANCH = "src.branch"
    SOURCE_CODE_COMMIT_HASH = "src.commit.hash"
    SOURCE_CODE_COMMIT_MESSAGE = "src.commit.message"