THUNDRA_MARKED_AS_SKIPPED = "thundra_marked_as_skipped"
THUNDRA_TEST_ALREADY_FINISHED = "thundra_test_already_finished"
THUNDRA_TEST_STARTED = "thundra_test_started"
THUNDRA_TEST_RESULTED = "thundra_test_resulted"
THUNDRA_TEST_FINISH_IN_HELPER = "thundra_test_finish_in_helper"
THUNDRA_FIXTURE_PREFIX = "x_thundra"
THUNDRA_SCOPE = "x-thundra-scope"